#include "dx_texture.h"

#include "dx_renderer.h"

dx_texture::dx_texture(const Eigen::Vector2i& size, texture_format format) : renderer_texture(format) {
    auto d3d_device = aorii::get_renderer<dx_renderer>()->get_d3d_device();
    D3D11_TEXTURE2D_DESC desc = {};
    desc.Width = size.x();
    desc.Height = size.y();
    desc.MipLevels = 1;
    desc.ArraySize = 1;
    desc.Format = get_dxgi_format(format);
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
    desc.Usage = D3D11_USAGE_DYNAMIC;
    desc.BindFlags = D3D11_BIND_SHADER_RESOURCE;
    desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
    desc.MiscFlags = 0;
    auto hr = d3d_device->CreateTexture2D(&desc, nullptr, &m_texture);
    if (FAILED(hr)) {
        spdlog::critical("无法创建纹理, 错误: {0}", hr);
        return;
    }

    D3D11_SHADER_RESOURCE_VIEW_DESC srv_desc = {};
    srv_desc.Format = desc.Format;
    srv_desc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;
    srv_desc.Texture2D.MostDetailedMip = 0;
    srv_desc.Texture2D.MipLevels = 1;
    hr = d3d_device->CreateShaderResourceView(m_texture, &srv_desc, &srv);
    if (FAILED(hr)) {
        spdlog::critical("无法创建着色器资源视图, 错误: {0}", hr);
        return;
    }
}

dx_texture::~dx_texture() {
    if (srv) srv->Release();
    if (m_texture) m_texture->Release();
}

void* dx_texture::lock(unsigned int* out_row_pitch) {
    D3D11_TEXTURE2D_DESC desc;
    m_texture->GetDesc(&desc);
    D3D11_MAPPED_SUBRESOURCE mappedResource;
    auto context = aorii::get_renderer<dx_renderer>()->get_d3d_context();
    context->Map(m_texture, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
    if (out_row_pitch) *out_row_pitch = mappedResource.RowPitch;
    return mappedResource.pData;
}

void dx_texture::unlock() {
    auto context = aorii::get_renderer<dx_renderer>()->get_d3d_context();
    context->Unmap(m_texture, 0);
}

bool dx_texture::resize(const Eigen::Vector2i& size) { return true; }

Eigen::Vector2i dx_texture::size() {
    D3D11_TEXTURE2D_DESC desc;
    m_texture->GetDesc(&desc);
    return { desc.Width, desc.Height };
}
